// The Santa Fe Stockmarket -- Implementation of the dividend process.

// -(int)setPeriod: (int)thePeriod
//	


#import "Dividend.h"
//#import "random.h"
#import <random.h>  //swarm library to get NormalDist
#include <math.h>
#include <misc.h>
#import "Interval.h"



@implementation Dividend
/*"
// This object produces a stochastic sequence of dividends.  The process
// is independent of the market and agents, depending only the parameters 
// that are set for the dividend process (and on the random number generator).
"*/

+createBegin: zone {
  Dividend *obj = [super createBegin: zone];

  obj->baseline = [Interval create];
  obj->mindividend = [Interval create];
  obj->maxdividend = [Interval create];
  obj->amplitude = [Interval create];
  obj->deviation = [Interval create];
  obj->rho = [Interval create];
  obj->gauss = [Interval create];
  obj->dvdnd = [Interval create];

  return obj;
}

+create: zone {
  Dividend *obj = [self createBegin: zone];

  obj = [obj createEnd];

  return obj;
}

/*"Creates a Swarm Normal Distribution object"*/
- initNormal: (int)seed
{
  
  normal=[NormalDist  create: [self getZone]  setGenerator: [MT19937gen create: [self getZone] setStateFromSeed: seed] setMean: 0 setVariance: 1];

  return self;
}

-setBaseline: (Interval *)theBaseline
{
  [baseline set: theBaseline];
  [theBaseline drop];
  return self;
}


-setmindividend: (Interval *)minimumDividend
{
  [mindividend set: minimumDividend];
  [minimumDividend drop];
  return self;
}


-setmaxdividend: (Interval *)maximumDividend
{
  [maxdividend set: maximumDividend];
  [maximumDividend drop];
  return self;
}


/*" Sets the "amplitude" parameter.    Returns the
//	value actually set, which may be clipped or rounded compared to the
//	supplied argument. See "-setDivType:".
"*/
-(Interval *)setAmplitude:(Interval *)theAmplitude
{
  [amplitude set: theAmplitude];
  if ([amplitude lt: [Interval constant: 0.0]]) 
    [amplitude set: [Interval constant: 0.0]];
  if ([amplitude gt: [Interval constant: 1.0]]) 
    [amplitude set: [Interval constant: 1.0]];
  [amplitude set: [[Interval constant: 0.0001] prod: [[[Interval constant: 10000.0] prod: amplitude] rint]]];
  [theAmplitude drop];
  return [[amplitude clone] ret];
}

/*" Sets the "period" parameter.   Returns the
// value actually set, which may be clipped compared to the supplied
// argument. See "-setDivType:". "*/

-(int)setPeriod: (int)thePeriod
{
  period = thePeriod;
  if (period < 2) 
    period = 2;
  return period;
}


-setDerivedParamsFlattenRho: (BOOL)flatten_rho gauss: (BOOL)flatten_gauss
/*
 * Sets various parameters derived from the externally-settable ones.  This
 * is called lazily, when a parameter is needed and the needsSetDerivedParams
 * flag is set.
 */
{
  [deviation set: [baseline prod: amplitude]];
// We round rho slightly for analytic ease
  [rho set: [[[Interval constant: -1.0]
	       quot: [Interval castInt: period]] exp]];
  [rho set: [[Interval constant: 0.0001]
	      prod: [[[Interval constant: 10000.0] prod: rho] rint]]];	
  if(flatten_rho) [rho flatten];
  [gauss set: [deviation prod: [[[Interval constant: 1.0]
				  diff: [rho squared]] sqrt]]];
  if(flatten_gauss) [gauss flatten];
  //pj:
  //dvdnd = baseline + gauss*normal();
  [dvdnd set: [baseline
		sum: [gauss
		       prod: [Interval constant: [normal getDoubleSample]]]]];
  return self;
}

/*" Returns the next value of the dividend.  This is the core method
  of the Dividend object, for which all else exists.  It does NOT use
  the global time, but simply assumes that one period passes between
  each call.  Note that "time" may not be the same as the global
  variable "t" because shifts are introduced to maintain phase when
  certain parameters are changed."*/

-(Interval *)dividend
{
  //pj:
  // dvdnd = baseline + rho*(dvdnd - baseline) + gauss*normal();
  [dvdnd set: [baseline
		sum: [[rho prod: [dvdnd diff: baseline]]
		       sum: [gauss
			      prod: [Interval
				      constant: [normal
						  getDoubleSample]]]]]]; 
  if ([dvdnd lt: mindividend]) 
    [dvdnd set: mindividend];
  if ([dvdnd gt: maxdividend]) 
    [dvdnd set: maxdividend];

  //   printf(" \n \n World dividend %f baseline %f rho %f max %f min  %f\n \n", dvdnd, baseline, rho, maxdividend, mindividend);

  return [[dvdnd clone] ret];
}

-(void)drop {
  [baseline drop];
  [amplitude drop];
  [mindividend drop];
  [maxdividend drop];
  [deviation drop];
  [rho drop];
  [gauss drop];
  [dvdnd drop];
  [super drop];
}

@end







