/*
    FEARLUS model0-7: Interval.h
    Copyright (C) 1999-2003  Macaulay Institute

    This file is part of FEARLUS model0-7, an agent-based model of land use
    change.

    FEARLUS model0-7 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-7 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/


#import <objectbase/SwarmObject.h>
#import <stdio.h>

@interface Interval : SwarmObject {
  double min;
  double max;
  const char *trace;
  @public
  double var;
  BOOL defined;
  BOOL positive;		// The number can be mathematically proven
				// to be always more than zero.
  BOOL negative;		// The number can be mathematically proven
				// to be always less than zero.
  BOOL constant;		// The number can only be set once.

  /* N.B. Nothing that gets put on the stack can ever be guaranteed positive,
     negative, or constant, so these will not be passed on to results */
}

+showWarnings;
+hideWarnings;
+fatalNaN;
+ignoreNaN;
+strictDivZero;
+ignoreDivZero;
+fatalUndef;
+ignoreUndef;

+(unsigned long)stackSize;
+(unsigned long)intervalCount;
+getZone;

+constant: (double)value;
+castInt: (int)value;
+castULL: (unsigned long long)value;
+create;
+createWith: (double)value;
+createWith: (double)value min: (double)lo max: (double)hi;
+createPositive;
+createPositiveWith: (double)value;
+createPositiveWith: (double)value min: (double)lo max: (double)hi;
+createNegative;
+createNegativeWith: (double)value;
+createNegativeWith: (double)value min: (double)lo max: (double)hi;
+createConstant;
+createConstantWith: (double)value;
+createConstantWith: (double)value min: (double)lo max: (double)hi;

-clone;
-trace: (const char *)label;
-untrace;

-bufread: (FILE *)fp;
-bufwrite;
-bufwrite: (FILE *)fp;
-bufwrite: (FILE *)fp format: (char *)format;
-(void)print;
-(void)print: (char *)format;
-(char *)printf: (char *)format;
-(char *)printfVar: (char *)format;
-(char *)printfRange: (char *)format;
+(void)printfclean;
-strread: (char *)buffer;
-(int)strwrite: (char *)buffer size: (size_t)bufsz;
-(int)strwriteVar: (char *)buffer size: (size_t)bufsz;
-(int)strwriteRange: (char *)buffer size: (size_t)bufsz;
-(int)strwrite: (char *)buffer size: (size_t)bufsz format: (char *)format;
-(int)strwriteVar: (char *)buffer size: (size_t)bufsz format: (char *)format;
-(int)strwriteRange: (char *)buffer size: (size_t)bufsz format: (char *)format;
-read: (int)fildes;
-write: (int)fildes;

-set: (Interval *)value;
-get: (double *)varptr min: (double *)minptr max: (double *)maxptr;
-(double)getVar;
-(double)getMin;
-(double)getMax;

/* Arithmetic methods */

-add: (Interval *)value;
-sum: (Interval *)value;
-inc;

-sub: (Interval *)value;
-diff: (Interval *)value;
-dec;

-mul: (Interval *)value;
-prod: (Interval *)value;

-div: (Interval *)value;
-quot: (Interval *)value;

-neg;
-(int)integer;
-rint;
-fabs;
-ceil;
-floor;

-(void)flatten;
-(BOOL)constrainMin: (Interval *)lo;
-(BOOL)constrainMax: (Interval *)hi;
-(BOOL)constrainPlusMinus: (Interval *)pm;

/* Monotone mathematical functions only */

-exp;
-expm1;
-sqrt;
-squared;

/* Comparison methods */

-(BOOL)eq: (Interval *)value;
-(BOOL)ne: (Interval *)value;
-(BOOL)gt: (Interval *)value;
-(BOOL)le: (Interval *)value;
-(BOOL)lt: (Interval *)value;
-(BOOL)ge: (Interval *)value;

-(BOOL)Meq: (Interval *)value;
-(BOOL)Mne: (Interval *)value;
-(BOOL)Mgt: (Interval *)value;
-(BOOL)Mle: (Interval *)value;
-(BOOL)Mlt: (Interval *)value;
-(BOOL)Mge: (Interval *)value;

-(BOOL)Leq: (Interval *)value;
-(BOOL)Lne: (Interval *)value;
-(BOOL)Lgt: (Interval *)value;
-(BOOL)Lle: (Interval *)value;
-(BOOL)Llt: (Interval *)value;
-(BOOL)Lge: (Interval *)value;

/* Interval comparison methods */

-(BOOL)contains: (Interval *)value;
-(BOOL)intersects: (Interval *)value;

-ret;
-call;
-fumble;
-(void)castVoid;
-(void)drop;

-(double)gap;

@end
